"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFunctionDefinitionHandler = exports.pushLayerARN = exports.applyTracingLayer = exports.applyExtensionLayer = exports.applyLambdaLibraryLayers = exports.findHandlers = exports.armRuntimeKeys = exports.runtimeLookup = exports.RuntimeType = void 0;
var RuntimeType;
(function (RuntimeType) {
    RuntimeType["NODE"] = "node";
    RuntimeType["PYTHON"] = "python";
    RuntimeType["DOTNET"] = "dotnet";
    RuntimeType["CUSTOM"] = "custom";
    RuntimeType["JAVA"] = "java";
    RuntimeType["RUBY"] = "ruby";
    RuntimeType["GO"] = "go";
    RuntimeType["UNSUPPORTED"] = "unsupported";
})(RuntimeType = exports.RuntimeType || (exports.RuntimeType = {}));
const X86_64_ARCHITECTURE = "x86_64";
const ARM64_ARCHITECTURE = "arm64";
const DEFAULT_ARCHITECTURE = X86_64_ARCHITECTURE;
const DEFAULT_REGION = "us-east-1";
exports.runtimeLookup = {
    "nodejs12.x": RuntimeType.NODE,
    "nodejs14.x": RuntimeType.NODE,
    "nodejs16.x": RuntimeType.NODE,
    "nodejs18.x": RuntimeType.NODE,
    "python3.7": RuntimeType.PYTHON,
    "python3.8": RuntimeType.PYTHON,
    "python3.9": RuntimeType.PYTHON,
    "python3.10": RuntimeType.PYTHON,
    "dotnetcore3.1": RuntimeType.DOTNET,
    dotnet6: RuntimeType.DOTNET,
    java11: RuntimeType.JAVA,
    java17: RuntimeType.JAVA,
    "java8.al2": RuntimeType.JAVA,
    java8: RuntimeType.JAVA,
    "provided.al2": RuntimeType.CUSTOM,
    provided: RuntimeType.CUSTOM,
    "ruby2.7": RuntimeType.RUBY,
    "ruby3.2": RuntimeType.RUBY,
    "go1.x": RuntimeType.GO,
};
exports.armRuntimeKeys = {
    "python3.8": "python3.8-arm",
    "python3.9": "python3.9-arm",
    "python3.10": "python3.10-arm",
    "ruby2.7": "ruby2.7-arm",
    "ruby3.2": "ruby3.2-arm",
    extension: "extension-arm",
    dotnet6: "dd-trace-dotnet-ARM",
};
function findHandlers(service, exclude, defaultRuntime) {
    return Object.entries(service.functions)
        .map(([name, handler]) => {
        let { runtime } = handler;
        if (runtime === undefined) {
            runtime = defaultRuntime;
        }
        if (runtime !== undefined && runtime in exports.runtimeLookup) {
            return { type: exports.runtimeLookup[runtime], runtime, name, handler };
        }
        return { type: RuntimeType.UNSUPPORTED, runtime, name, handler };
    })
        .filter((result) => result !== undefined)
        .filter((result) => exclude === undefined || (exclude !== undefined && !exclude.includes(result.name)));
}
exports.findHandlers = findHandlers;
function applyLambdaLibraryLayers(service, handlers, layers, accountId) {
    var _a, _b, _c;
    const { region } = service.provider;
    // It's possible a local account layer is being used in a region we have not published to so we use a default region's ARNs
    const shouldUseDefaultRegion = layers.regions[region] === undefined && accountId !== undefined;
    const regionRuntimes = shouldUseDefaultRegion ? layers.regions[DEFAULT_REGION] : layers.regions[region];
    if (regionRuntimes === undefined) {
        return;
    }
    for (const handler of handlers) {
        if (handler.type === RuntimeType.UNSUPPORTED) {
            continue;
        }
        const { runtime } = handler;
        if (runtime === undefined) {
            continue;
        }
        let runtimeKey = runtime;
        const architecture = (_c = (_b = (_a = handler.handler) === null || _a === void 0 ? void 0 : _a.architecture) !== null && _b !== void 0 ? _b : service.provider.architecture) !== null && _c !== void 0 ? _c : DEFAULT_ARCHITECTURE;
        const isArm64 = architecture === ARM64_ARCHITECTURE;
        if (isArm64 && runtime in exports.armRuntimeKeys) {
            runtimeKey = exports.armRuntimeKeys[runtime];
            const prevLayerARN = accountId !== undefined
                ? buildLocalLambdaLayerARN(regionRuntimes[runtime], accountId, region)
                : regionRuntimes[runtime];
            removePreviousLayer(service, handler, prevLayerARN);
        }
        let layerARN = regionRuntimes[runtimeKey];
        if (accountId && layerARN) {
            layerARN = buildLocalLambdaLayerARN(layerARN, accountId, region);
        }
        if (layerARN) {
            addLayer(service, handler, layerARN);
        }
    }
}
exports.applyLambdaLibraryLayers = applyLambdaLibraryLayers;
function applyExtensionLayer(service, handlers, layers, accountId) {
    var _a, _b;
    const { region } = service.provider;
    // It's possible a local account layer is being used in a region we have not published to so we use a default region's ARNs
    const shouldUseDefaultRegion = layers.regions[region] === undefined && accountId !== undefined;
    const regionRuntimes = shouldUseDefaultRegion ? layers.regions[DEFAULT_REGION] : layers.regions[region];
    if (regionRuntimes === undefined) {
        return;
    }
    for (const handler of handlers) {
        if (handler.type === RuntimeType.UNSUPPORTED) {
            continue;
        }
        const architecture = (_b = (_a = handler.handler.architecture) !== null && _a !== void 0 ? _a : service.provider.architecture) !== null && _b !== void 0 ? _b : DEFAULT_ARCHITECTURE;
        let extensionLayerKey = "extension";
        if (architecture === ARM64_ARCHITECTURE) {
            const prevExtensionARN = accountId !== undefined
                ? buildLocalLambdaLayerARN(regionRuntimes[extensionLayerKey], accountId, region)
                : regionRuntimes[extensionLayerKey];
            removePreviousLayer(service, handler, prevExtensionARN);
            extensionLayerKey = exports.armRuntimeKeys[extensionLayerKey];
        }
        let extensionARN = regionRuntimes[extensionLayerKey];
        if (accountId && extensionARN) {
            extensionARN = buildLocalLambdaLayerARN(extensionARN, accountId, region);
        }
        if (extensionARN) {
            addLayer(service, handler, extensionARN);
        }
    }
}
exports.applyExtensionLayer = applyExtensionLayer;
function applyTracingLayer(service, handler, layers, runtimeKey, accountId) {
    const { region } = service.provider;
    // It's possible a local account layer is being used in a region we have not published to so we use a default region's ARNs
    const shouldUseDefaultRegion = layers.regions[region] === undefined && accountId !== undefined;
    const regionRuntimes = shouldUseDefaultRegion ? layers.regions[DEFAULT_REGION] : layers.regions[region];
    if (regionRuntimes === undefined) {
        return;
    }
    let traceLayerARN = regionRuntimes[runtimeKey];
    if (accountId && traceLayerARN) {
        traceLayerARN = buildLocalLambdaLayerARN(traceLayerARN, accountId, region);
    }
    if (traceLayerARN) {
        addLayer(service, handler, traceLayerARN);
    }
}
exports.applyTracingLayer = applyTracingLayer;
function pushLayerARN(layerARN, currentLayers) {
    const layerSet = new Set(currentLayers);
    layerSet.add(layerARN);
    return Array.from(layerSet);
}
exports.pushLayerARN = pushLayerARN;
function isFunctionDefinitionHandler(funcDef) {
    return typeof funcDef.handler === "string";
}
exports.isFunctionDefinitionHandler = isFunctionDefinitionHandler;
function addLayer(service, handler, layerArn) {
    setLayers(handler, pushLayerARN(layerArn, getLayers(service, handler)));
}
function getLayers(service, handler) {
    const functionLayersList = handler.handler.layers || [];
    const serviceLayersList = service.provider.layers || [];
    // Function-level layers override service-level layers
    // Append to the function-level layers if other function-level layers are present
    // If service-level layers are present
    // Set them at the function level, as our layers are runtime-dependent and could vary
    // between functions in the same project
    if (functionLayersList.length > 0 || serviceLayersList.length === 0) {
        return functionLayersList;
    }
    else {
        return serviceLayersList;
    }
}
function removePreviousLayer(service, handler, previousLayer) {
    let layersList = getLayers(service, handler);
    if (new Set(layersList).has(previousLayer)) {
        layersList = layersList === null || layersList === void 0 ? void 0 : layersList.filter((layer) => layer !== previousLayer);
    }
    setLayers(handler, layersList);
}
function setLayers(handler, layers) {
    handler.handler.layers = layers;
}
function buildLocalLambdaLayerARN(layerARN, accountId, region) {
    if (layerARN === undefined) {
        return;
    }
    // Rebuild the layer ARN to use the given account's region and partition
    const [layerName, layerVersion] = layerARN.split(":").slice(6, 8);
    const partition = getAwsPartitionByRegion(region);
    const localLayerARN = `arn:${partition}:lambda:${region}:${accountId}:layer:${layerName}:${layerVersion}`;
    return localLayerARN;
}
function getAwsPartitionByRegion(region) {
    if (region.startsWith("us-gov-")) {
        return "aws-us-gov";
    }
    if (region.startsWith("cn-")) {
        return "aws-cn";
    }
    return "aws";
}
//# sourceMappingURL=layer.js.map